# Generated by Django 5.1.1 on 2024-09-15 22:17

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("test_app", "0005_alter_order_options_alter_product_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="order",
            options={
                "permissions": (
                    (
                        "bulk_delete%(class)s",
                        "Can delete multiple objects from %(class)s at once",
                    ),
                    (
                        "bulk_change%(class)s",
                        "Can update multiple objects from %(class)s at once",
                    ),
                )
            },
        ),
        migrations.AlterModelOptions(
            name="product",
            options={
                "permissions": (
                    (
                        "bulk_delete%(class)s",
                        "Can delete multiple objects from %(class)s at once",
                    ),
                    (
                        "bulk_change%(class)s",
                        "Can update multiple objects from %(class)s at once",
                    ),
                )
            },
        ),
    ]
